package com.jzlife.report.controller;


import com.github.pagehelper.PageInfo;
import com.jzlife.report.anno.LogExportFiled;
import com.jzlife.report.service.newreport.InvestmentReportService;
import com.jzlife.report.util.ExcelPOIUtils;
import com.jzlife.report.util.ResponseUtil;
import com.jzlife.report.vo.DayReportBaseVo;
import com.jzlife.report.vo.ImportStationInfoVo;
import com.jzlife.report.vo.InvestmentAndResignVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Api(value = "招商数据看板", description = "招商数据看板")
@RestController
@RequestMapping(value = "report")
public class InvestmentReportController {

    @Autowired
    private InvestmentReportService investmentReportService;


    @ApiOperation(value = "招商情况和离职情况及各来源入职情况", httpMethod = "POST", notes = "时间格式为：yyyy-MM-dd HH:mm:ss")
    @PostMapping("investmentAndResignList")
    @LogExportFiled(funName = "招商看板")
    public Object investmentAndResignList(@Validated @RequestBody InvestmentAndResignVo vo) {
        String errorMsg = checkChooseTime(vo.getStartTime(), vo.getEndTime());
        if (StringUtils.isNotBlank(errorMsg)) {
            return ResponseUtil.fail400(errorMsg);
        }
        Map<String, Object> data = investmentReportService.investmentAndResignList(vo);
        return ResponseUtil.ok(data);
    }

    private String checkChooseTime(String startTime, String endTime) {
        LocalDate start = LocalDate.parse(startTime, DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        LocalDate end = LocalDate.parse(endTime, DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        if (start.isAfter(end) || start.isEqual(end)) {
            return "开始时间不可大于或等于结束时间！";
        }
        if (end.getYear() - start.getYear() > 1) {
            return "所选时间跨度不可超过1年";
        }
        if (start.isAfter(LocalDate.now().minusDays(1))) {
            return "仅支持查看T-1数据";
        }
        return "";
    }


    @ApiOperation(value = "导入离职原因&跟单流失原因", httpMethod = "POST", notes = "导入离职原因&跟单流失原因")
    @PostMapping("importResignAndLossReason")
    @LogExportFiled(funName = "导入离职原因&跟单流失原因")
    public Object importResignAndLossReason(@RequestParam("file") MultipartFile file) {
        try {
            //错误maps
            Map<Integer, String> errorMap = new HashMap<>();
            //错误列表
            List<ImportStationInfoVo> errors = new ArrayList<>();
            //站点数据列表
            List<ImportStationInfoVo> stations = ExcelPOIUtils.readExc(file, ImportStationInfoVo.class, errorMap, errors, 6, 1);
            //保存录入信息
            Map<String, Object> result = investmentReportService.importData(stations);
            HashMap<String, Object> maps = new HashMap<>();
            //在成功的列表中去除失败的数据
            if (result.containsKey("fail")) {
                List<String> of = (List<String>) result.get("fail");
                maps.put("otherFail", of);
            }
            maps.put("succ", stations);
            maps.put("fail", errorMap);
            return ResponseUtil.ok("导入成功", maps);
        } catch (Exception e) {
            return ResponseUtil.fail400(e.getMessage());
        }
    }

    @ApiOperation(value = "查看离职&跟单流失原因", httpMethod = "GET", notes = "查看离职&跟单流失原因")
    @GetMapping("getResignAndLossReason")
    @LogExportFiled(funName = "离职&跟单流失看板")
    public Object getResignAndLossReason(@RequestParam(value = "type", defaultValue = "1,2")
                                         @ApiParam(name = "查询类型", value = "1离职 2流失", defaultValue = "1,2") String type) {
        return ResponseUtil.ok(investmentReportService.getResignAndLossReason(type));
    }


    @ApiOperation(value = "招商明细", httpMethod = "POST", notes = "时间格式为：yyyy-MM-dd,查询选定时间段内的每日招商明细")
    @PostMapping("dayInvestMenDetail")
    @LogExportFiled(funName = "招商明细")
    public Object dayInvestMenDetail(@Validated @RequestBody DayReportBaseVo vo) {
        PageInfo<Map<String, Object>> data = investmentReportService.dayInvestMentDetail(vo);
        return ResponseUtil.ok(data);
    }
}
