package com.jzlife.report.controller;

import com.github.pagehelper.PageInfo;
import com.jzlife.report.anno.LogExportFiled;
import com.jzlife.report.dao.hmorder.SalesOrderMapper;
import com.jzlife.report.dto.BuzLineKewordDto;
import com.jzlife.report.dto.export.SalesReportExport;
import com.jzlife.report.dto.export.ServiceGmvExport;
import com.jzlife.report.schedule.TaskReportData;
import com.jzlife.report.service.hmorder.SalesOrderExportService;
import com.jzlife.report.service.newreport.PrivilegeManService;
import com.jzlife.report.service.newreport.SalesReportService;
import com.jzlife.report.service.nurse.CityStationService;
import com.jzlife.report.util.ExcelUtils;
import com.jzlife.report.util.ResponseUtil;
import com.jzlife.report.util.SortCityUtil;
import com.jzlife.report.util.UserUtil;
import com.jzlife.report.vo.DayReportBaseVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@Api(value = "销售数据报表", description = "销售数据报表")
@RequestMapping("salesdata")
@RestController
public class SalesDataReportController {

	@Autowired
	SalesReportService salesReportService;

	@Autowired
	SalesOrderMapper salesOrderMapper;

	@Autowired
	SalesOrderExportService salesOrderExportService;

	@Autowired
	TaskReportData task;

	@Autowired
	CityStationService cityStationService;

	@Autowired
	UserUtil userUtil;
	
	@Autowired
	PrivilegeManService privilegeManService;

	@ApiOperation("刷新销售缓存数据")
	@GetMapping("refCacheSale")
	public Object refCacheSale(String startMonth, String endMonth) {
		new Thread() {
			public void run() {
				task.insertSalesCache(startMonth, endMonth);
			};
		}.start();
		return ResponseUtil.ok();
	}

	@ApiOperation("公司销售GMV目标")
	@GetMapping("getCSGmvTarget")
	@LogExportFiled(funName = "公司销售GMV目标")
	public Object getCompanySalesGmvTarget(HttpServletRequest request) {
		Integer userId = userUtil.getUser(request);
		if (userId == null) {
			return ResponseUtil.fail402();
		}
		List<String> userCity = cityStationService.getUserCity(userId);
		if (null == userCity || userCity.size() == 0) {
			return ResponseUtil.fail402();
		}
		String city = String.join(",", userCity);
		
		boolean boo = privilegeManService.getPrivilegeManByUserId(userId);
		if(boo) {
			city=null;
		}

		Map<String, Object> data = salesReportService.getCompanySalesGmvTarget(city);
		return ResponseUtil.ok(data);
	}

	@ApiOperation("公司销售GMV目标和复购情况 2021-06")
	@GetMapping("getCSGmvData")
	@LogExportFiled(funName = "公司销售GMV目标和复购情况")
	public Object getCompanySalesGmvData(String yearMonth, HttpServletRequest request) {
		Integer userId = userUtil.getUser(request);
		if (userId == null) {
			return ResponseUtil.fail402();
		}
		List<String> userCity = cityStationService.getUserCity(userId);
		if (null == userCity || userCity.size() == 0) {
			return ResponseUtil.fail402();
		}

		if (StringUtils.isBlank(yearMonth)) {
			return ResponseUtil.fail("请选择月份");
		}
		String city = String.join(",", userCity);
		String sortCity = SortCityUtil.sortCity(city);
		boolean boo = privilegeManService.getPrivilegeManByUserId(userId);
		if(boo) {//全部城市权限，不要带城市，查所有
			sortCity=null;
		}
		
		Map<String, Object> companySalesGmvData = salesReportService.getCompanySalesGmvData(yearMonth, sortCity);

		return ResponseUtil.ok(companySalesGmvData);
	}

	@ApiOperation("实销额与单量  timeType 1天2月")
	@GetMapping("getCSTrendchart")
	@LogExportFiled(funName = "实销额与单量")
	public Object getCSTrendchart(String channel,String city, String businessTypeName, String businessLine, Integer timeType,
			String startTime, String endTime, String startMonth, String endMonth) {

		if (StringUtils.isBlank(businessTypeName)) {
//			businessTypeName = "保洁,保姆,品类,搬家,附加,普通保洁单次,普通保洁周期,鲸选保洁单次,鲸选保洁周期";
			businessTypeName = "保洁,保姆,品类,搬家,附加";
			// 普通保洁单次,普通保洁周期,鲸选保洁单次,鲸选保洁周期 作为 businessLine
		}

		if (StringUtils.isBlank(city)) {
			return ResponseUtil.fail("请选择城市");
		}

		if (timeType == 1 && (StringUtils.isBlank(startTime) || StringUtils.isBlank(endTime))) {
			return ResponseUtil.fail("请选择日期");
		}
		if (timeType == 2 && (StringUtils.isBlank(startMonth) || StringUtils.isBlank(endMonth))) {
			return ResponseUtil.fail("请选择月份");
		}

		Map<String, Object> data = salesReportService.getCompanySalesbrokenLine(channel,city, businessTypeName, businessLine,
				timeType, startTime, endTime, startMonth, endMonth);

		return ResponseUtil.ok(data);
	}

	@ApiOperation("服务GMV与单量 timeType 1天2月")
	@GetMapping("getOrderNumberchart")
	@LogExportFiled(funName = "服务GMV与单量")
	public Object getOrderNumberchart(String city, String businessTypeName, Integer timeType, String startTime,
			String endTime, String startMonth, String endMonth) {

		if (StringUtils.isBlank(businessTypeName)) {
//			businessTypeName = "保洁,保姆,品类,搬家,附加,普通保洁单次,普通保洁周期,鲸选保洁单次,鲸选保洁周期";
			businessTypeName = "保洁,品类";
			// 普通保洁单次,普通保洁周期,鲸选保洁单次,鲸选保洁周期 作为 businessLine
		}

		if (StringUtils.isBlank(city)) {
			return ResponseUtil.fail("请选择城市");
		}

		if (timeType == 1 && (StringUtils.isBlank(startTime) || StringUtils.isBlank(endTime))) {
			return ResponseUtil.fail("请选择日期");
		}
		if (timeType == 2 && (StringUtils.isBlank(startMonth) || StringUtils.isBlank(endMonth))) {
			return ResponseUtil.fail("请选择月份");
		}

		Map<String, Object> data = salesReportService.getOrderNumberchart(city, businessTypeName, timeType, startTime,
				endTime, startMonth, endMonth);

		return ResponseUtil.ok(data);
	}

	@ApiOperation("获取来源渠道--订单")
	@GetMapping("getChannel")
	public Object getChannel() {
		List<String> list = salesOrderMapper.getChannel();
		return ResponseUtil.ok(list);
	}

	@ApiOperation("获取保姆来源")
	@GetMapping("getNurseSource")
	public Object getNurseSource() {
		List<String> list = salesOrderMapper.getNurseSource();
		return ResponseUtil.ok(list);
	}

	@ApiOperation("小程序订单列表")
	@GetMapping("selectLittleAppOrder")
	@LogExportFiled(funName = "小程序订单列表")
	public Object selectLittleAppOrder(@RequestParam(defaultValue = "1") Integer pageNo,
			@RequestParam(defaultValue = "50") Integer pageSize, String startDate, String endDate, String channel,
			String city, String levelOne) {
		if (StringUtils.isBlank(startDate) || StringUtils.isBlank(endDate)) {
			return ResponseUtil.fail("时间范围不能为空");
		}
		if (StringUtils.isBlank(city)) {
			return ResponseUtil.fail("请选择城市");
		}

		startDate = startDate + " 00:00:00";
		endDate = endDate + " 23:59:59";
		return salesOrderExportService.selectLittleAppOrder(pageNo, pageSize, startDate, endDate, channel, city,
				levelOne);
	}

	@ApiOperation("各渠道销售明细列表")
	@GetMapping("selectChannelSales")
	@LogExportFiled(funName = "各渠道销售明细列表")
	public Object selectChannelSales(@RequestParam(defaultValue = "1") Integer pageNo,
			@RequestParam(defaultValue = "50") Integer pageSize, String startDate, String endDate, String channel,
			String city, String levelOne) {
		startDate = startDate + " 00:00:00";
		endDate = endDate + " 23:59:59";
		if (StringUtils.isBlank(city)) {
			return ResponseUtil.fail("请选择城市");
		}

		return salesOrderExportService.selectChannelSales(pageNo, pageSize, startDate, endDate, channel, city,
				levelOne);
	}

	@ApiOperation("销售汇总列表")
	@GetMapping("salesSummary")
	@LogExportFiled(funName = "销售汇总列表")
	public Object salesSummary(@RequestParam(defaultValue = "1") Integer pageNo,
			@RequestParam(defaultValue = "50") Integer pageSize, String startDate, String endDate, String channel,
			String city, String levelOne) {
		startDate = startDate + " 00:00:00";
		endDate = endDate + " 23:59:59";
		if (StringUtils.isBlank(city)) {
			return ResponseUtil.fail("请选择城市");
		}
		return salesOrderExportService.salesSummary(pageNo, pageSize, startDate, endDate, channel, city, levelOne);
	}

	@ApiOperation("销售汇总导出")
	@GetMapping("salesSummaryExport")
	@LogExportFiled(funName = "销售汇总导出")
	public void salesSummaryExport(String startDate, String endDate, String channel, String city, String levelOne,
			HttpServletResponse response) throws Exception {
		startDate = startDate + " 00:00:00";
		endDate = endDate + " 23:59:59";
		if (StringUtils.isBlank(city)) {
			return;
		}
		List<SalesReportExport> list = salesOrderExportService.salesSummaryExport(startDate, endDate, channel, city,
				levelOne);

		ExcelUtils.exportExcel(list, null, "Sheet1", SalesReportExport.class, "销售汇总.xls", response);
	}

	@ApiOperation("服务GMV汇总")
	@GetMapping("serviceGMVCount")
	@LogExportFiled(funName = "服务GMV汇总")
	public Object serviceGMVCount(@RequestParam(defaultValue = "1") Integer pageNo,
			@RequestParam(defaultValue = "50") Integer pageSize, String startDate, String endDate, String city,
			String levelOne) {
		startDate = startDate + " 00:00:00";
		endDate = endDate + " 23:59:59";
		if (StringUtils.isBlank(city)) {
			return ResponseUtil.fail("请选择城市");
		}
		return salesOrderExportService.serviceGMVCount(pageNo, pageSize, startDate, endDate, city, levelOne);
	}

	@ApiOperation("服务GMV汇总导出")
	@GetMapping("serviceGMVCountExport")
	@LogExportFiled(funName = "服务GMV汇总导出")
	public void serviceGMVCountExport(String startDate, String endDate, String city, String levelOne,
			HttpServletResponse response) throws Exception {
		startDate = startDate + " 00:00:00";
		endDate = endDate + " 23:59:59";
		if (StringUtils.isBlank(city)) {
			return;
		}

		List<ServiceGmvExport> list = salesOrderExportService.serviceGMVCountExport(startDate, endDate, city, levelOne);
		ExcelUtils.exportExcel(list, null, "Sheet1", ServiceGmvExport.class, "服务GMV汇总.xls", response);
	}

	@ApiOperation("月销售进度回溯（每日）")
	@GetMapping("monthSalesPlan")
	@LogExportFiled(funName = "月销售进度回溯（每日）")
	public Object monthSalesPlan(@RequestParam(defaultValue = "1") Integer pageNo,
			@RequestParam(defaultValue = "50") Integer pageSize, String startDate, String city, String levelOne) {
		startDate = startDate + " 00:00:00";
		String endDate = startDate + " 23:59:59";// String endDate,
		if (StringUtils.isBlank(city)) {
			return ResponseUtil.fail("请选择城市");
		}
		return salesOrderExportService.monthSalesPlan(pageNo, pageSize, startDate, endDate, city, levelOne);
	}

	@ApiOperation("非盒马渠道当月实销额扇形图")
	@GetMapping("notHMActualSales")
	@LogExportFiled(funName = "非盒马渠道当月实销额扇形图")
	public Object notHMActualSales(String startDate, String endDate) {
		if (StringUtils.isBlank(startDate) || StringUtils.isBlank(endDate)) {
			return ResponseUtil.fail("时间不能为空");
		}
		startDate = startDate + " 00:00:00";
		endDate = endDate + " 23:59:59";
		return salesOrderExportService.notHMActualSales(startDate, endDate);
	}

	@ApiOperation("线索列表")
	@GetMapping("selectClue")
	@LogExportFiled(funName = "线索列表")
	public Object selectClue(@RequestParam(defaultValue = "1") Integer pageNo,
			@RequestParam(defaultValue = "50") Integer pageSize, String startDate, String endDate, String resource,
			String city, String serviceModel) {
		startDate = startDate + " 00:00:00";
		endDate = endDate + " 23:59:59";

		if (StringUtils.isBlank(city)) {
			return ResponseUtil.fail("请选择城市");
		}

		return salesOrderExportService.selectClue(pageNo, pageSize, startDate, endDate, resource, city, serviceModel);
	}

	@ApiOperation("保姆签单")
	@GetMapping("servantSignOrder")
	@LogExportFiled(funName = "保姆签单")
	public Object servantSignOrder(@RequestParam(defaultValue = "1") Integer pageNo,
			@RequestParam(defaultValue = "50") Integer pageSize, String startDate, String endDate, String city,
			String remark) {
		startDate = startDate + " 00:00:00";
		endDate = endDate + " 23:59:59";

		if (StringUtils.isBlank(city)) {
			return ResponseUtil.fail("请选择城市");
		}
		return salesOrderExportService.servantSignOrder(pageNo, pageSize, startDate, endDate, city, remark);
	}

	@ApiOperation("支付记录")
	@GetMapping("cluePayOrder")
	@LogExportFiled(funName = "支付记录")
	public Object cluePayOrder(@RequestParam(defaultValue = "1") Integer pageNo,
			@RequestParam(defaultValue = "50") Integer pageSize, String startDate, String endDate) {
		startDate = startDate + " 00:00:00";
		endDate = endDate + " 23:59:59";
		return salesOrderExportService.cluePayOrder(pageNo, pageSize, startDate, endDate);
	}

	@ApiOperation(value = "BD转化", httpMethod = "POST", notes = "时间格式为：yyyy-MM-dd,查询选定时间段内的BD转化")
	@PostMapping("bdInvertDetail")
	@LogExportFiled(funName = "BD转化")
	public Object bdInvertDetail(@Validated @RequestBody DayReportBaseVo vo) {
		PageInfo<Map<String, Object>> data = salesReportService.bdInvertDetail(vo);
		return ResponseUtil.ok(data);
	}

	@ApiOperation("单品走势")
	@GetMapping("orderTrend")
	@LogExportFiled(funName = "单品走势")
	public Object orderTrend(String startDate, String endDate, String city, String levelOne, String keyWord) {
		if (StringUtils.isBlank(city)) {
			return ResponseUtil.fail("请选择城市");
		}
		return salesOrderExportService.danpingzoushi(startDate, endDate, city, levelOne, keyWord);
	}

	@ApiOperation("单品走势下拉框，业务线，关键字")
	@GetMapping("orderTrendSelect")
	public Object orderTrendSelect() {
		List<BuzLineKewordDto> buzLine = salesOrderExportService.getBuzLine();
		return ResponseUtil.ok(buzLine);
	}

	@ApiOperation("城市进度,某天,日期不能为空")
	@GetMapping("cityProgress")
	@LogExportFiled(funName = "城市进度")
	public Object cityProgress(String date) {
		if (StringUtils.isBlank(date)) {
			return ResponseUtil.fail("日期不能为空");
		}
		return salesOrderExportService.cityProgress(date);
	}

}
