package com.jzlife.report.controller;


import com.github.pagehelper.PageInfo;
import com.jzlife.report.anno.LogExportFiled;
import com.jzlife.report.dao.nurse.Report30daysProductStocksResultMapper;
import com.jzlife.report.dto.Report30daysProductStocks;
import com.jzlife.report.dto.Report30daysProductStocksResult;
import com.jzlife.report.service.newreport.ServicePowerReportService;
import com.jzlife.report.util.ExcelPOIUtils;
import com.jzlife.report.util.ResponseUtil;
import com.jzlife.report.util.excel.ExcelXlsxReader;
import com.jzlife.report.vo.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.InputStream;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.temporal.TemporalAdjusters;
import java.util.*;

@Api(value = "服务力看板", description = "服务力看板")
@RestController
@RequestMapping(value = "report")
public class ServicePowerReportController {


    @Autowired
    private ServicePowerReportService service;

    @Autowired
    Report30daysProductStocksResultMapper daysProductStocksResultMapper;


    @ApiOperation(value = "服务力多Y轴看板", httpMethod = "POST", notes = "时间格式为：yyyy-MM")
    @PostMapping("servicePowerReport")
    @LogExportFiled(funName = "服务力多Y轴看板")
    public Object servicePowerReport(@Validated @RequestBody ReportBaseVo vo) {
        String errorMsg = checkChooseTime(vo.getStartTime(), vo.getEndTime());
        if (StringUtils.isNotBlank(errorMsg)) {
            return ResponseUtil.fail400(errorMsg);
        }
        Map<String, Object> data = service.servicePowerReport(vo);
        return ResponseUtil.ok(data);
    }

    @ApiOperation(value = "服务力薪资分布散点图", httpMethod = "POST", notes = "时间格式为：yyyy-MM")
    @PostMapping("servicePowerSalaryReport")
    @LogExportFiled(funName = "服务力薪资分布散点图")
    public Object servicePowerSalaryReport(@Validated @RequestBody ReportBaseVo vo) {
        String errorMsg = checkChooseTime(vo.getStartTime(), vo.getEndTime());
        if (StringUtils.isNotBlank(errorMsg)) {
            return ResponseUtil.fail400(errorMsg);
        }
        Map<String, Object> data = service.servicePowerSalaryReport(vo);
        return ResponseUtil.ok(data);
    }


    private String checkChooseTime(String startTime, String endTime) {
        LocalDate start = LocalDate.parse(startTime + "-01", DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        LocalDate end = LocalDate.parse(endTime + "-01", DateTimeFormatter.ofPattern("yyyy-MM-dd")).with(TemporalAdjusters.lastDayOfMonth());
        if (start.isAfter(end) || start.isEqual(end)) {
            return "开始时间不可大于或等于结束时间！";
        }
        if (end.getYear() - start.getYear() > 1) {
            return "所选时间跨度不可超过1年";
        }
        if (start.getYear() + 2 < LocalDate.now().getYear()) {
            return "仅支持统计最近两年的数据";
        }
        return "";
    }


    @ApiOperation(value = "导入各城市14天库存率", httpMethod = "POST", notes = "导入各城市14天库存率")
    @PostMapping("importCity14DaysStockRadio")
    @LogExportFiled(funName = "导入各城市14天库存率")
    public Object importCity14DaysStackRadio(@RequestParam("file") MultipartFile file) {
        try {
            //错误maps
            Map<Integer, String> errorMap = new HashMap<>();
            //错误列表
            List<DaysStackRadioVo> errors = new ArrayList<>();
            //站点数据列表
            List<DaysStackRadioVo> stations = ExcelPOIUtils.readExc(file, DaysStackRadioVo.class, errorMap, errors, 11, 2);
            //保存录入信息
            Map<String, Object> result = service.importData(stations);
            HashMap<String, Object> maps = new HashMap<>();
            //在成功的列表中去除失败的数据
            if (result.containsKey("fail")) {
                List<String> of = (List<String>) result.get("fail");
                maps.put("otherFail", of);
            }
            maps.put("succ", stations);
            maps.put("fail", errorMap);
            return ResponseUtil.ok("导入成功", maps);
        } catch (Exception e) {
            return ResponseUtil.fail400(e.getMessage());
        }
    }


    @ApiOperation(value = "导入近30天商品库存", httpMethod = "POST", notes = "导入近30天商品库存")
    @PostMapping("import30DaysProductStock")
    @LogExportFiled(funName = "导入近30天商品库存")
    public Object import30DaysProductStock(@RequestParam("file") MultipartFile file, HttpServletRequest request) {
        Integer userId = (Integer) request.getAttribute("userId");
        String s = LocalDate.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")) + "_userId_" + userId + "_01";
        Report30daysProductStocksResult result1 = daysProductStocksResultMapper.selectBySerialNo(s);
        if (null != result1) {
            return ResponseUtil.fail400("今天已成功导入过，请勿重复导入！");
        }
        try {
            new Thread(() -> {
                {
                    try {
                        List<Report30daysProductStocks> productStocks = readExcelXlsx(file.getInputStream());
                        service.import30DaysProductStock(productStocks, s);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }).start();
            HashMap<String, Object> maps = new HashMap<>();
            maps.put("serialNo", s);
            Report30daysProductStocksResult result = new Report30daysProductStocksResult();
            result.setSerialNo(s);
            result.setResultType(2);
            result.setCreateTime(new Date());
            daysProductStocksResultMapper.insert(result);
            return ResponseUtil.ok("后台导入中,请稍后再查看", maps);
        } catch (Exception e) {
            return ResponseUtil.fail400(e.getMessage());
        }
    }


    @ApiOperation(value = "查询导入30天商品库存结果", httpMethod = "GET", notes = "查询导入30天商品库存结果")
    @GetMapping("import30DaysProductStockResult")
    @LogExportFiled(funName = "查询导入30天商品库存结果")
    public Object import30DaysProductStockResult(@RequestParam("serialNo") String serialNo) {
        return service.get30DaysProductStockResult(serialNo);
    }


    @ApiOperation(value = "服务力工单明细", httpMethod = "POST", notes = "时间格式为：yyyy-MM-dd,使用 盒马服务站")
    @PostMapping("servicePowerTicketDetail")
    @LogExportFiled(funName = "服务力工单明细")
    public Object servicePowerTicketDetail(@Validated @RequestBody DayReportBaseVo vo) {
        PageInfo<Map<String, Object>> data = service.servicePowerTicketDetail(vo);
        return ResponseUtil.ok(data);
    }

    @ApiOperation(value = "服务力工单汇总", httpMethod = "POST", notes = "时间格式为：yyyy-MM-dd 使用盒马服务站")
    @PostMapping("servicePowerTicketInfo")
    @LogExportFiled(funName = "服务力工单汇总")
    public Object servicePowerTicketInfo(@Validated @RequestBody DayReportBaseVo vo) {
        PageInfo<Map<String, Object>> data = service.servicePowerTicketInfo(vo);
        return ResponseUtil.ok(data);
    }

    @ApiOperation(value = "服务力商家信息", httpMethod = "POST", notes = "时间格式为：yyyy-MM-dd 使用自家服务站")
    @PostMapping("servicePowerServerInfo")
    @LogExportFiled(funName = "服务力商家信息")
    public Object servicePowerServerInfo(@Validated @RequestBody DayReportBaseVo vo) {
        PageInfo<Map<String, Object>> data = service.servicePowerServerInfo(vo);
        return ResponseUtil.ok(data);
    }

    @ApiOperation(value = "服务力当月人员汇总", httpMethod = "POST", notes = "时间格式为：yyyy-MM-dd 使用自家服务站")
    @PostMapping("servicePowerStationPersonInfo")
    @LogExportFiled(funName = "服务力当月人员汇总")
    public Object servicePowerStationPersonInfo(@Validated @RequestBody DayReportBaseVo vo) {
        PageInfo<Map<String, Object>> data = service.servicePowerStationPersonInfo(vo);
        return ResponseUtil.ok(data);
    }

    @ApiOperation(value = "住宿费", httpMethod = "POST", notes = "时间格式为：yyyy-MM 使用自家服务站")
    @PostMapping("servicePowerRentFee")
    @LogExportFiled(funName = "住宿费")
    public Object servicePowerRentFee(@Validated @RequestBody ReportBaseVo vo) {
        PageInfo<Map<String, Object>> data = service.servicePowerRentFee(vo);
        return ResponseUtil.ok(data);
    }

    @ApiOperation(value = "宿舍长补贴", httpMethod = "POST", notes = "时间格式为：yyyy-MM 使用自家服务站")
    @PostMapping("servicePowerWornSubsidies")
    @LogExportFiled(funName = "宿舍长补贴")
    public Object servicePowerWornSubsidies(@Validated @RequestBody ReportBaseVo vo) {
        PageInfo<Map<String, Object>> data = service.servicePowerWornSubsidies(vo);
        return ResponseUtil.ok(data);
    }

    @ApiOperation(value = "师傅奖励", httpMethod = "POST", notes = "时间格式为：yyyy-MM 使用自家服务站")
    @PostMapping("servicePowerTeacherReward")
    @LogExportFiled(funName = "师傅奖励")
    public Object servicePowerTeacherReward(@Validated @RequestBody ReportBaseVo vo) {
        PageInfo<Map<String, Object>> data = service.servicePowerTeacherReward(vo);
        return ResponseUtil.ok(data);
    }

    @ApiOperation(value = "跟单奖励", httpMethod = "POST", notes = "时间格式为：yyyy-MM-dd 使用自家服务站")
    @PostMapping("servicePowerOrderReward")
    @LogExportFiled(funName = "跟单奖励")
    public Object servicePowerOrderReward(@Validated @RequestBody DayReportBaseVo vo) {
        PageInfo<Map<String, Object>> data = service.servicePowerOrderReward(vo);
        return ResponseUtil.ok(data);
    }

    public static List<Report30daysProductStocks> readExcelXlsx(InputStream in) throws Exception {
        ExcelXlsxReader excelXls = new ExcelXlsxReader();
        List<List<String>> dataList = excelXls.process(in);
        List<Report30daysProductStocks> stations = new ArrayList<>();
        for (int i = 0; i < dataList.size(); i++) {
            if (i > 0) {
                List<String> e = dataList.get(i);
                Report30daysProductStocks aa = new Report30daysProductStocks();
                aa.setStockDate(e.get(0));
                aa.setSupplier(e.get(1));
                aa.setShopName(e.get(2));
                aa.setSku(Long.valueOf(e.get(3)));
                aa.setProductId(Long.valueOf(e.get(4)));
                aa.setProductName(e.get(5));
                aa.setTimeStamp(e.get(6));
                aa.setTotalCount(Integer.parseInt(e.get(7)));
                aa.setFreeCount(Integer.parseInt(e.get(8)));
                stations.add(aa);
            }
        }
        return stations;
    }


    // 服务力三期新增
    @ApiOperation(value = "服务力14天库存", httpMethod = "GET", notes = " t-1 ~ t+12")
    @GetMapping("service14DayStock")
    @LogExportFiled(funName = "服务力14天库存")
    public Object service14DayStock(HttpServletRequest request) {
        List<Map<String, Object>> data = service.service14DayStock(request.getHeader("userId"));
        return ResponseUtil.ok(data);
    }

    @ApiOperation(value = "服务力指标&服务站指标", httpMethod = "GET", notes = "yyyy-MM 格式")
    @GetMapping("serviceGoalData")
    @LogExportFiled(funName = "服务力指标&服务站指标")
    public Object serviceGoalData(@RequestParam("month") String month, HttpServletRequest request) {
        String userId = request.getHeader("userId");
        Map<String, Object> data = service.serviceGoalData(month,userId);
        return ResponseUtil.ok(data);
    }


    @ApiOperation(value = "服务站与门店库存", httpMethod = "POST", notes = "使用盒马服务站")
    @PostMapping("stationAndShopStock")
    @LogExportFiled(funName = "服务站与门店库存")
    public Object stationAndShopStock(@Validated @RequestBody ReportStationAndShopVo vo) {
        Map<String, Object> data = service.stationAndShopStock(vo);
        return ResponseUtil.ok(data);
    }

    @ApiOperation(value = "订单热力图", httpMethod = "POST", notes = "使用盒马服务站")
    @PostMapping("orderHeatMap")
    @LogExportFiled(funName = "订单热力图")
    public Object orderHeatMap(@Validated @RequestBody BaseRequestVo vo) {
        Map<String, Object> data = service.createOrderHeatMap(vo);
        return ResponseUtil.ok(data);
    }


    @ApiOperation(value = "保洁人均单量&品类人均工时", httpMethod = "POST", notes = "使用盒马服务站")
    @PostMapping("CleanPerCapitalOrderQuality")
    @LogExportFiled(funName = "保洁人均单量&品类人均工时")
    public Object CleanPerCapitalOrderQuality(@Validated @RequestBody ReportCleanPerCaptchaOrderQualityVo vo) {
        if (vo.getTimeType() == 1 && (StringUtils.isEmpty(vo.getStartMonth()) || StringUtils.isEmpty(vo.getEndMonth()))) {
            return ResponseUtil.fail400("开始结束月份不可为空");
        }
        if (vo.getTimeType() == 2 && (StringUtils.isEmpty(vo.getStartTime()) || StringUtils.isEmpty(vo.getEndTime()))) {
            return ResponseUtil.fail400("开始结束日期不可为空");
        }
        return service.cleanPerCapitalOrderQuality(vo);
    }
}
