package com.jzlife.report.controller;


import com.github.pagehelper.PageInfo;
import com.jzlife.report.anno.LogExportFiled;
import com.jzlife.report.service.hmorder.StationSerevice;
import com.jzlife.report.service.newreport.ServiceQualityReportService;
import com.jzlife.report.util.ResponseUtil;
import com.jzlife.report.vo.DayReportBaseVo;
import com.jzlife.report.vo.ReportBaseVo;
import com.jzlife.report.vo.ReportCustomerClaimAndPowerVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.temporal.TemporalAdjusters;
import java.util.Map;

@Api(value = "服务质量看板", description = "服务质量看板")
@RestController
@RequestMapping(value = "report")
public class ServiceQualityReportController {

    @Autowired
    private ServiceQualityReportService service;

    @Autowired
    StationSerevice stationSerevice;


    @ApiOperation(value = "服务质量情况堆叠折线图", httpMethod = "POST", notes = "时间格式为：yyyy-MM")
    @PostMapping("serviceQualityStackLine")
    @LogExportFiled(funName = "服务质量情况堆叠折线图")
    public Object serviceQualityStackLine(@Validated @RequestBody ReportBaseVo vo) {
        String errorMsg = checkChooseTime(vo.getStartTime(), vo.getEndTime());
        if (StringUtils.isNotBlank(errorMsg)) {
            return ResponseUtil.fail400(errorMsg);
        }
        Map<String, Object> data = service.serviceQualityStackLine(vo);
        return ResponseUtil.ok(data);
    }


    @ApiOperation(value = "客诉数和服务能力折线图", httpMethod = "POST", notes = "时间可选，在颗粒度为月份时格式为yyyy-MM，在颗粒度为天时 为 yyyy-MM-dd ")
    @PostMapping("customerClaimAndServicePowerLineGraph")
    @LogExportFiled(funName = "客诉数和服务能力折线图")
    public Object customerClaimAndServicePowerLineGraph(@Validated @RequestBody ReportCustomerClaimAndPowerVo vo) {
        if (vo.getTimeType() == 1 && (StringUtils.isEmpty(vo.getStartMonth()) || StringUtils.isEmpty(vo.getEndMonth()))) {
            return ResponseUtil.fail400("开始结束月份不可为空");
        }
        if (vo.getTimeType() == 2 && (StringUtils.isEmpty(vo.getStartTime()) || StringUtils.isEmpty(vo.getEndTime()))) {
            return ResponseUtil.fail400("开始结束日期不可为空");
        }
        Map<String, Object> data = service.customerClaimAndServicePowerLineGraph(vo);
        return ResponseUtil.ok(data);
    }


    private String checkChooseTime(String startTime, String endTime) {
        LocalDate start = LocalDate.parse(startTime + "-01", DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        LocalDate end = LocalDate.parse(endTime + "-01", DateTimeFormatter.ofPattern("yyyy-MM-dd")).with(TemporalAdjusters.lastDayOfMonth());
        if (start.isAfter(end) || start.isEqual(end)) {
            return "开始时间不可大于或等于结束时间！";
        }
        if (end.getYear() - start.getYear() > 1) {
            return "所选时间跨度不可超过1年";
        }
        if (start.getYear() + 2 < LocalDate.now().getYear()) {
            return "仅支持统计最近两年的数据";
        }
        return "";
    }

    @ApiOperation(value = "客诉一级指标", httpMethod = "GET", notes = "包含进线接听率、外呼接听率、现存客诉单、平均处理时效、S级工单、>48小时工单")
    @GetMapping("claimSummaryAndPie")
    @LogExportFiled(funName = "客诉一级指标")
    public Object claimSummaryAndPie() {
        Map<String, Object> data = service.claimSummaryAndPie();
        return ResponseUtil.ok(data);
    }


    @ApiOperation(value = "客诉明细", httpMethod = "POST", notes = "时间格式为：yyyy-MM-dd,查询选定时间段内的每日客诉明细 使用盒马服务站")
    @PostMapping("dayCustomerClaimDetail")
    @LogExportFiled(funName = "客诉明细")
    public Object dayCustomerClaimDetail(@Validated @RequestBody DayReportBaseVo vo) {
        PageInfo<Map<String, Object>> data = service.dayCustomerClaimDetail(vo);
        return ResponseUtil.ok(data);
    }

    @ApiOperation(value = "S级工单明细", httpMethod = "GET", notes = "查询未关闭的S级工单")
    @GetMapping("superTicketDetail")
    @LogExportFiled(funName = "S级工单明细")
    public Object superTicketDetail(@RequestParam(defaultValue = "1") Integer pageNo,
                                    @RequestParam(defaultValue = "20") Integer pageSize) {
        PageInfo<Map<String, Object>> data = service.superTicketDetail(pageNo, pageSize);
        return ResponseUtil.ok(data);
    }


    @ApiOperation(value = "48小时工单明细", httpMethod = "GET", notes = "查询48小时未关闭的工单")
    @GetMapping("aliveTwoDayTicketDetail")
    @LogExportFiled(funName = "48小时工单明细")
    public Object aliveTwoDayTicketDetail(@RequestParam(defaultValue = "1") Integer pageNo,
                                          @RequestParam(defaultValue = "20") Integer pageSize) {
        PageInfo<Map<String, Object>> data = service.aliveTwoDayTicketDetail(pageNo, pageSize);
        return ResponseUtil.ok(data);
    }


    @ApiOperation(value = "在线客服报表-明细")
    @GetMapping("onlineChatSummary")
    @LogExportFiled(funName = "在线客服报表-明细")
    public Object onlineChatSummary(@RequestParam(defaultValue = "1") Integer pageNo,
                                    @RequestParam(defaultValue = "50") Integer pageSize, String startDate, String endDate, String customerId, String userName) {

        startDate = startDate + " 00:00:00";
        endDate = endDate + " 23:59:59";

        PageInfo<Map<String, Object>> data = service.onlineChatSummary(pageNo, pageSize, startDate, endDate, customerId, userName);
        return ResponseUtil.ok(data);
    }
    
    @ApiOperation(value = "在线客服报表-汇总")
    @GetMapping("onlineChathz")
    @LogExportFiled(funName = "在线客服报表-汇总")
    public Object onlineChathz(@RequestParam(defaultValue = "1") Integer pageNo,
                                    @RequestParam(defaultValue = "50") Integer pageSize, String startDate, String endDate, String userName) {

        PageInfo<Map<String, Object>> data = service.onlineChathz(pageNo, pageSize, startDate, endDate, userName);
        return ResponseUtil.ok(data);
    }

}
