package com.jzlife.report.service.newreport.impl;

import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jzlife.report.dao.manager.EntryResignLossRecordMapper;
import com.jzlife.report.dao.manager.TempInvestInfoMapper;
import com.jzlife.report.dto.EntryResignLossRecord;
import com.jzlife.report.service.newreport.InvestmentReportService;
import com.jzlife.report.vo.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.*;
import java.util.stream.Collectors;


@Service
@Slf4j
public class InvestmentReportServiceImpl implements InvestmentReportService {


    @Resource
    private TempInvestInfoMapper investInfoMapper;

    @Resource
    private EntryResignLossRecordMapper resignLossRecordMapper;


    @Override
    public Map<String, Object> investmentAndResignList(InvestmentAndResignVo vo) {
        Map<String, Object> result = new HashMap<>();
        Map<String, Object> temp1 = investInfoMapper.selectByCityOrStation(vo);

        List<String> category = new ArrayList<>();
        Map<String, Long> map1 = investInfoMapper.selectAllSourceNoTrainInfoByCityOrStation(vo);
        Map<String, Object> map2 = investInfoMapper.selectAllSourceSignInfoByCityOrStation(vo);
        map1.entrySet().stream().sorted(Map.Entry.comparingByValue()).forEach(e -> category.add(e.getKey()));
        result.put("category", category);

        InvestmentVo investmentVo = new InvestmentVo();
        investmentVo.setName("未跟单人数");
        List<Long> noTran = new ArrayList<>();
        for (String s : category) {
            Long aLong = map1.get(s);
            noTran.add(aLong);
        }
        investmentVo.setData(noTran);

        InvestmentVo investmentVo1 = new InvestmentVo();
        investmentVo1.setName("入职人数");
        List<Long> signs = new ArrayList<>();
        for (String out : category) {
            Long aLong = (Long) map2.get(out);
            signs.add(aLong);
        }
        investmentVo1.setData(signs);
        List<InvestmentVo> investmentVos = Arrays.asList(investmentVo, investmentVo1);
        result.put("series", investmentVos);

        //流失率 当月解约 /（月初人数 + 当月入职） *100%
        //当月离职人数
        DecimalFormat decimalFormat = new DecimalFormat("##.00%");
        Long bjresignCount = (Long) temp1.get("保洁离职人数");
        Long plresignCount = (Long) temp1.get("品类离职人数");
        Long bjsignCount = (Long) temp1.get("保洁入职人数");
        Long plsignCount = (Long) temp1.get("品类入职人数");
        Map<String, Long> workingCounts = investInfoMapper.getWorkingCountByStartTimeAndCityOrStation(vo);
        Long bjCount = workingCounts.get("bjCount");
        Long plCount = workingCounts.get("plCount");
        if (plCount == 0L) {
            temp1.put("品类流失率", "NOData");
        } else if (plresignCount == 0L) {
            temp1.put("品类流失率", "0.00%");
        } else {
            temp1.put("品类流失率", decimalFormat.format(new BigDecimal(plresignCount).divide(new BigDecimal(plCount + plsignCount), 4, RoundingMode.HALF_UP)));
        }
        if (bjCount == 0L) {
            temp1.put("保洁流失率", "NOData");
        } else if (bjresignCount == 0L) {
            temp1.put("保洁流失率", "0.00%");
        } else {
            temp1.put("保洁流失率", decimalFormat.format(new BigDecimal(bjresignCount).divide(new BigDecimal(bjCount + bjsignCount), 4, RoundingMode.HALF_UP)));
        }
        result.put("conversion", temp1);
        return result;
    }

    @Override
    public Map<String, Object> importData(List<ImportStationInfoVo> stations) {
        Map<String, Object> returnMap = new HashMap<>();
        List<String> success = new ArrayList<>();
        List<String> fail = new ArrayList<>();
        //检查对应的保姆是否存在
        for (int i = stations.size() - 1; i >= 0; i--) {
            ImportStationInfoVo importStationInfoVo = stations.get(i);
            if (investInfoMapper.checkWorkerExist(importStationInfoVo.getWorkId()) == 0) {
                fail.add(importStationInfoVo.getWorkId());
                stations.remove(i);
            } else {
                //检查是否已导入，同一个work_phone仅可导入一次
                int count = resignLossRecordMapper.selectCountByWorkPhone(importStationInfoVo.getWorkPhone());
                if (count < 1) {
                    success.add(importStationInfoVo.getWorkId());
                } else {
                    fail.add(importStationInfoVo.getWorkId());
                    stations.remove(i);
                }
            }
        }
        Map<Integer, List<ImportStationInfoVo>> collect = stations.stream().collect(Collectors.groupingBy(importStationInfoVo -> stations.size() / 1000));
        Set<Integer> integers = collect.keySet();
        integers.forEach(a -> resignLossRecordMapper.batchInsert(collect.get(a)));
        returnMap.put("success", success);
        returnMap.put("fail", fail);
        return returnMap;
    }

    @Override
    public Object getResignAndLossReason(String type) {
        List<EntryResignLossRecord> entryResignLossRecords;
        if (StringUtils.equals(type, "1,2") || StringUtils.isEmpty(type)) {
            entryResignLossRecords = resignLossRecordMapper.selectAll();
        } else {
            entryResignLossRecords = resignLossRecordMapper.selectByType(type);
        }
        JSONObject jsonObject = new JSONObject();
        Map<String, Integer> hash1 = new HashMap<>();
        Map<String, Integer> hash2 = new HashMap<>();
        Map<String, List<EntryResignLossRecord>> collect1 = entryResignLossRecords.stream()
                .filter(a -> a.getType() == 1).collect(Collectors.groupingBy(EntryResignLossRecord::getLevelOne));
        Map<String, List<EntryResignLossRecord>> collect2 = entryResignLossRecords.stream()
                .filter(a -> a.getType() == 2).collect(Collectors.groupingBy(EntryResignLossRecord::getLevelOne));
        collect1.forEach((a, b) -> hash1.put(a, collect1.get(a).size()));
        collect2.forEach((a, b) -> hash2.put(a, collect2.get(a).size()));
        Map<String, Integer> result1 = new LinkedHashMap<>();
        Map<String, Integer> result2 = new LinkedHashMap<>();
        hash1.entrySet().stream()
                .sorted(Map.Entry.<String, Integer>comparingByKey()
                        .reversed()).forEachOrdered(e -> result1.put(e.getKey(), e.getValue()));
        hash2.entrySet().stream()
                .sorted(Map.Entry.<String, Integer>comparingByKey()
                        .reversed()).forEachOrdered(e -> result2.put(e.getKey(), e.getValue()));
        if (StringUtils.equals(type, "1,2") || StringUtils.isEmpty(type)) {
            LossAndResignVo vo = new LossAndResignVo();
            vo.setType(2);
            vo.setCountMap(tranMap(result2));
            jsonObject.put("loss", vo);
            LossAndResignVo vo1 = new LossAndResignVo();
            vo1.setType(1);
            vo1.setCountMap(tranMap(result1));
            jsonObject.put("resign", vo1);
        } else {
            if (StringUtils.equals(type, "2")) {
                LossAndResignVo vo = new LossAndResignVo();
                vo.setType(2);
                vo.setCountMap(tranMap(result2));
                jsonObject.put("loss", vo);
            } else if (StringUtils.equals(type, "1")) {
                LossAndResignVo vo1 = new LossAndResignVo();
                vo1.setType(1);
                vo1.setCountMap(tranMap(result1));
                jsonObject.put("resign", vo1);
            }
        }
        return jsonObject;
    }

    @Override
    public PageInfo<Map<String, Object>> dayInvestMentDetail(DayReportBaseVo vo) {
        PageHelper.startPage(vo.getStartPage(), vo.getLimit());
        List<Map<String, Object>> mapList = investInfoMapper.dayInvestMentDetail(vo);
        return new PageInfo<>(mapList);
    }

    private Map<String, Object> tranMap(Map<String, Integer> result2) {
        Map<String, Object> temp = new HashMap<>();
        for (String key : result2.keySet()) {
            String means = resignLossRecordMapper.getMeansByValue(key);
            temp.put(means, result2.get(key));
        }
        return temp;
    }
}
